#import "ArpSniffer.h"
#import "ArpController.h"

//#define EXTENDED_DEVIVES


@implementation ArpController

NSLock * arpLock;

// message called at startup time
- (void) awakeFromNib
{	
	// add network devices to drop down list and select the first device
	[devices removeAllItems];
	[devices addItemWithTitle:@"en0 - Ethernet"];
	[devices addItemWithTitle:@"en1 - Wireless"];
#ifdef EXTENDED_DEVIVES
	[devices addItemWithTitle:@"en2 - Other"];
	[devices addItemWithTitle:@"en3 - Other"];
#endif
	[devices selectItemAtIndex:0];
	
	// refresh the table, should be empty
	arplist = [[NSMutableArray alloc] initWithCapacity:30];	
	[table reloadData];
	
	// initialize the lock object, used for access to arplist array
	arpLock = [[NSConditionLock alloc] init];
	
	@try
	{
		// intialize the sniffer core
		arpSniffer = [[ArpSniffer alloc] initialize:@"en0"];
	}
	@catch (NSException * ex)
	{
		NSAlert * alert = [NSAlert alertWithMessageText:@"Error opening network device"
						defaultButton:@"OK"
					  alternateButton:nil
						  otherButton:nil
			informativeTextWithFormat:[ex reason]];
		[alert runModal];
		return;
	}
	
	[NSAlert alertWithMessageText:@"Foo"
					defaultButton:@"OK"
				  alternateButton:nil
					  otherButton:nil
		informativeTextWithFormat:@"Test"];
	
	// TODO - investigate initWithCapacity
	
	// start the sniffer thread
	[NSThread detachNewThreadSelector: @selector(startCapture:)
							 toTarget: arpSniffer withObject: self];

}

// message called at quit time
- (void) applicationWillTerminate:(NSNotification *)notification
{
	// release objects
	[arpLock release];
	[arpSniffer stopCapture];
	[arpSniffer release];
	[arplist release];
}

// clear the arp cache and refresh the screen
- (IBAction) pressButton:(id)sender
{
	[arpLock lock];
	[arplist removeAllObjects];
	[arpLock unlock];
	
	[table reloadData];
	
	return;
}


// clear the arp cache and refresh the screen
- (IBAction) deviceSelected:(id)sender
{	
	NSString * device;
	[arpSniffer stopCapture];
	[arpSniffer release];
		
	// clear all items
	[arpLock lock];
	[arplist removeAllObjects];
	[arpLock unlock];
	[table reloadData];
	
	// select device from drop down list
	switch ([devices indexOfSelectedItem])
	{
		case 0:
			device = @"en0";
			break;
		case 1:
			device = @"en1";
			break;
#ifdef EXTENDED_DEVIVES
		case 2:
			device = @"en2";
			break;
		case 3:
			device = @"en3";
			break;
#endif
		default:
			// this should never happen
			
			return;
			break;
	}

	printf("deviceSelected()\n");
	
	@try
	{
		arpSniffer = [[ArpSniffer alloc] initialize:device];
	}
	@catch (NSException * ex)
	{
		NSAlert * alert = [NSAlert alertWithMessageText:@"Error opening network device"
										  defaultButton:@"OK"
										alternateButton:nil
											otherButton:nil
							  informativeTextWithFormat:[ex reason]];
		[alert runModal];
		return;
	}
	
	printf("deviceSelected(): Sniff Thread\n");
	
	// start the sniffer thread
	[NSThread detachNewThreadSelector: @selector(startCapture:)
							 toTarget: arpSniffer withObject: self];
	
	printf("deviceSelected(): Thread Started\n");
	
	return;
}


// incoming arp request received
- (void)arpReceived:(char *)ipAddress macAddress:(char *)macAddress;
{
	NSString * keys[2];
	NSString * values[2];
	
	keys[0] = @"IP Address";
	values[0] = [NSString stringWithFormat:@"%s", ipAddress];
		
	keys[1] = @"MAC Address";
	values[1] = [NSString stringWithFormat:@"%s", macAddress];
	
	NSDictionary * entry = [NSDictionary dictionaryWithObjects:(id *)values
										forKeys:(id *)keys count:2];

	// iterate through the existing arp cache, look for duplicate entries
	int i;
	for (i = 0; i < [arplist count]; i++)
	{
		NSDictionary * item = [arplist objectAtIndex:i];
		NSString * ip = [item objectForKey:@"IP Address"];
		NSString * mac = [item objectForKey:@"MAC Address"];
		if ([ip isEqualToString:values[0]] && [mac isEqualToString:values[1]])
		{
			// duplicate entry found, bail out
			return;
		}
		if ([ip isEqualToString:values[0]] || [mac isEqualToString:values[1]])
		{
			// we have a change in matchup between IP and MAC
			// TODO - notify? color?
			
			// ignore for now.
		}
	}
	
	
	// TODO - check for duplicates
	[arpLock lock];
	[arplist addObject: entry];
	[arpLock unlock];
	
	[table reloadData];
	
	return;
}

// TABLE DATA SOURCE
- (int) numberOfRowsInTableView: (NSTableView *)table
{
	[arpLock lock];
	int count = [arplist count];
	[arpLock unlock];

	// returns number of items in array
	return count;
}

// this message is called for each row of the table
- (id) tableView: (NSTableView *)table objectValueForTableColumn:(NSTableColumn*)col
			row:(int)rowIndex
{
	NSDictionary * rowData;

	[arpLock lock];
	rowData = [ arplist objectAtIndex:rowIndex ];
	[arpLock unlock];
	
	return [rowData objectForKey:[col identifier]];
}

@end